#!/usr/bin/env python
import argparse
import os
import sys
import warnings

from django.core.management import execute_from_command_line

os.environ['DJANGO_SETTINGS_MODULE'] = 'wagtailmenus.settings.testing'


def make_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '--deprecation',
        choices=['all', 'pending', 'imminent', 'none'],
        default='imminent'
    )
    return parser


def parse_args(args=None):
    return make_parser().parse_known_args(args)


def runtests():
    parsed_args, unparsed_args = parse_args()

    only_wagtailmenus = r'^wagtailmenus(\.|$)'
    if parsed_args.deprecation == 'all':
        # Show all deprecation warnings from all packages
        warnings.simplefilter('default', category=DeprecationWarning)
        warnings.simplefilter('default', category=PendingDeprecationWarning)
    elif parsed_args.deprecation == 'pending':
        # Show all deprecation warnings
        warnings.filterwarnings('default', category=DeprecationWarning, module=only_wagtailmenus)
        warnings.filterwarnings('default', category=PendingDeprecationWarning, module=only_wagtailmenus)
    elif parsed_args.deprecation == 'imminent':
        # Show only imminent deprecation warnings
        warnings.filterwarnings('default', category=DeprecationWarning, module=only_wagtailmenus)
    elif parsed_args.deprecation == 'none':
        # Deprecation warnings are ignored
        pass

    argv = [sys.argv[0], 'test'] + unparsed_args
    return execute_from_command_line(argv)

if __name__ == '__main__':
    sys.exit(runtests())



from django.template import Context

from wagtailmenus.models import MainMenu
from wagtailmenus.tests import utils
from wagtailmenus.tests.test_mainmenu_class import MainMenuTestCase


class TestCreateDictFromParentContext(MainMenuTestCase):

    # ------------------------------------------------------------------------
    # Menu.create_dict_from_parent_context()
    # ------------------------------------------------------------------------

    def get_render_ready_menu_instance(self, parent_context=None):
        menu = MainMenu.objects.get(pk=1)
        ctx_vals = utils.make_contextualvals_instance(parent_context=parent_context)
        opt_vals = utils.make_optionvals_instance()
        menu.prepare_to_render(ctx_vals.request, ctx_vals, opt_vals)
        return menu

    def test_flattens_context_if_parent_context_is_django_template_context(self):
        menu = self.get_render_ready_menu_instance()
        template_context = menu._contextual_vals.parent_context

        result = menu.create_dict_from_parent_context()
        self.assertIsNot(result, template_context)
        self.assertEqual(result, template_context.flatten())

    def test_copies_value_if_parent_context_is_dict(self):
        dict_context = {'blah': 'blah'}
        menu = self.get_render_ready_menu_instance(parent_context=dict_context)

        result = menu.create_dict_from_parent_context()
        self.assertIsNot(result, dict_context)
        self.assertEqual(result, dict_context)

    def test_returns_empty_dict_if_partent_context_is_some_other_value_type(self):
        simple_context = Context({'blah': 'blah'})
        menu = self.get_render_ready_menu_instance(parent_context=simple_context)

        result = menu.create_dict_from_parent_context()
        self.assertIsNot(result, simple_context)
        self.assertIsInstance(result, dict)


class TestGetMenuItemsForRendering(MainMenuTestCase):

    # ------------------------------------------------------------------------
    # Menu.get_menu_items_for_rendering()
    # ------------------------------------------------------------------------

    def get_render_ready_menu_instance(self, **option_vals):
        menu = MainMenu.objects.get(pk=1)
        ctx_vals = utils.make_contextualvals_instance()
        opt_vals = utils.make_optionvals_instance(**option_vals)
        menu.prepare_to_render(ctx_vals.request, ctx_vals, opt_vals)
        return menu

    def test_sub_menu_items_not_added_inline_by_default(self):
        menu = self.get_render_ready_menu_instance()
        items_with_children = tuple(
            item for item in menu.get_menu_items_for_rendering()
            if getattr(item, 'has_children_in_menu', False)
        )
        self.assertGreater(len(items_with_children), 1)
        for menu_item in items_with_children:
            self.assertFalse(hasattr(menu_item, 'sub_menu'))

    def test_sub_menu_items_added_inline_if_option_value_set_to_true(self):
        menu = self.get_render_ready_menu_instance(add_sub_menus_inline=True)
        items_with_children = tuple(
            item for item in menu.get_menu_items_for_rendering()
            if getattr(item, 'has_children_in_menu', False)
        )
        for menu_item in items_with_children:
            self.assertTrue(hasattr(menu_item, 'sub_menu'))
            self.assertIsInstance(menu_item.sub_menu, menu.get_sub_menu_class())



from django.template import Library
from wagtail.core.models import Page

from wagtailmenus.conf import constants, settings
from wagtailmenus.errors import SubMenuUsageError
from wagtailmenus.utils.misc import validate_supplied_values

register = Library()


def split_if_string(val, separator=','):
    if isinstance(val, str):
        return tuple(item.strip() for item in val.split(separator))
    return val


@register.simple_tag(takes_context=True)
def main_menu(
    context, max_levels=None, use_specific=None, apply_active_classes=True,
    allow_repeating_parents=True, show_multiple_levels=True,
    template='', sub_menu_template='', sub_menu_templates=None,
    use_absolute_page_urls=False, add_sub_menus_inline=None, **kwargs
):
    validate_supplied_values('main_menu', max_levels=max_levels,
                             use_specific=use_specific)

    if not show_multiple_levels:
        max_levels = 1

    menu_class = settings.models.MAIN_MENU_MODEL
    return menu_class.render_from_tag(
        context=context,
        max_levels=max_levels,
        use_specific=use_specific,
        apply_active_classes=apply_active_classes,
        allow_repeating_parents=allow_repeating_parents,
        use_absolute_page_urls=use_absolute_page_urls,
        add_sub_menus_inline=add_sub_menus_inline,
        template_name=template,
        sub_menu_template_name=sub_menu_template,
        sub_menu_template_names=split_if_string(sub_menu_templates),
        **kwargs
    )


@register.simple_tag(takes_context=True)
def flat_menu(
    context, handle, max_levels=None, use_specific=None,
    show_menu_heading=False, apply_active_classes=False,
    allow_repeating_parents=True, show_multiple_levels=True,
    template='', sub_menu_template='', sub_menu_templates=None,
    fall_back_to_default_site_menus=None, use_absolute_page_urls=False,
    add_sub_menus_inline=None, **kwargs
):
    validate_supplied_values('flat_menu', max_levels=max_levels,
                             use_specific=use_specific)

    if fall_back_to_default_site_menus is None:
        fall_back_to_default_site_menus = settings.FLAT_MENUS_FALL_BACK_TO_DEFAULT_SITE_MENUS

    if not show_multiple_levels:
        max_levels = 1

    menu_class = settings.models.FLAT_MENU_MODEL
    return menu_class.render_from_tag(
        context=context,
        handle=handle,
        fall_back_to_default_site_menus=fall_back_to_default_site_menus,
        max_levels=max_levels,
        use_specific=use_specific,
        apply_active_classes=apply_active_classes,
        allow_repeating_parents=allow_repeating_parents,
        use_absolute_page_urls=use_absolute_page_urls,
        add_sub_menus_inline=add_sub_menus_inline,
        template_name=template,
        sub_menu_template_name=sub_menu_template,
        sub_menu_template_names=split_if_string(sub_menu_templates),
        show_menu_heading=show_menu_heading,
        **kwargs
    )


@register.simple_tag(takes_context=True)
def section_menu(
    context, show_section_root=True, show_multiple_levels=True,
    apply_active_classes=True, allow_repeating_parents=True,
    max_levels=settings.DEFAULT_SECTION_MENU_MAX_LEVELS,
    template='', sub_menu_template='', sub_menu_templates=None,
    use_specific=settings.DEFAULT_SECTION_MENU_USE_SPECIFIC,
    use_absolute_page_urls=False, add_sub_menus_inline=None, **kwargs
):
    """Render a section menu for the current section."""
    validate_supplied_values('section_menu', max_levels=max_levels,
                             use_specific=use_specific)

    if not show_multiple_levels:
        max_levels = 1

    menu_class = settings.objects.SECTION_MENU_CLASS
    return menu_class.render_from_tag(
        context=context,
        max_levels=max_levels,
        use_specific=use_specific,
        apply_active_classes=apply_active_classes,
        allow_repeating_parents=allow_repeating_parents,
        use_absolute_page_urls=use_absolute_page_urls,
        add_sub_menus_inline=add_sub_menus_inline,
        template_name=template,
        sub_menu_template_name=sub_menu_template,
        sub_menu_template_names=split_if_string(sub_menu_templates),
        show_section_root=show_section_root,
        **kwargs
    )


@register.simple_tag(takes_context=True)
def children_menu(
    context, parent_page=None, allow_repeating_parents=True,
    apply_active_classes=False,
    max_levels=settings.DEFAULT_CHILDREN_MENU_MAX_LEVELS,
    template='', sub_menu_template='', sub_menu_templates=None,
    use_specific=settings.DEFAULT_CHILDREN_MENU_USE_SPECIFIC,
    use_absolute_page_urls=False, add_sub_menus_inline=None, **kwargs
):
    validate_supplied_values(
        'children_menu', max_levels=max_levels, use_specific=use_specific,
        parent_page=parent_page)

    menu_class = settings.objects.CHILDREN_MENU_CLASS
    return menu_class.render_from_tag(
        context=context,
        parent_page=parent_page,
        max_levels=max_levels,
        use_specific=use_specific,
        apply_active_classes=apply_active_classes,
        allow_repeating_parents=allow_repeating_parents,
        use_absolute_page_urls=use_absolute_page_urls,
        add_sub_menus_inline=add_sub_menus_inline,
        template_name=template,
        sub_menu_template_name=sub_menu_template,
        sub_menu_template_names=split_if_string(sub_menu_templates),
        **kwargs
    )


@register.simple_tag(takes_context=True)
def sub_menu(
    context, menuitem_or_page, use_specific=None, allow_repeating_parents=None,
    apply_active_classes=None, template='', use_absolute_page_urls=None,
    add_sub_menus_inline=None, **kwargs
):
    """
    Retrieve the children pages for the `menuitem_or_page` provided, turn them
    into menu items, and render them to a template.
    """
    validate_supplied_values('sub_menu', use_specific=use_specific,
                             menuitem_or_page=menuitem_or_page)

    max_levels = context.get(
        'max_levels', settings.DEFAULT_CHILDREN_MENU_MAX_LEVELS
    )

    if use_specific is None:
        use_specific = context.get(
            'use_specific', constants.USE_SPECIFIC_AUTO)

    if apply_active_classes is None:
        apply_active_classes = context.get('apply_active_classes', True)

    if allow_repeating_parents is None:
        allow_repeating_parents = context.get('allow_repeating_parents', True)

    if use_absolute_page_urls is None:
        use_absolute_page_urls = context.get('use_absolute_page_urls', False)

    if add_sub_menus_inline is None:
        add_sub_menus_inline = context.get('add_sub_menus_inline', False)

    if isinstance(menuitem_or_page, Page):
        parent_page = menuitem_or_page
    else:
        parent_page = menuitem_or_page.link_page

    original_menu = context.get('original_menu_instance')
    if original_menu is None:
        raise SubMenuUsageError()

    menu_class = original_menu.get_sub_menu_class()
    return menu_class.render_from_tag(
        context=context,
        parent_page=parent_page,
        max_levels=max_levels,
        use_specific=use_specific,
        apply_active_classes=apply_active_classes,
        allow_repeating_parents=allow_repeating_parents,
        use_absolute_page_urls=use_absolute_page_urls,
        add_sub_menus_inline=add_sub_menus_inline,
        template_name=template,
        **kwargs
    )



from __future__ import absolute_import, unicode_literals

from bs4 import BeautifulSoup
from django.test import TestCase
from wagtail.core import hooks


class TestHooks(TestCase):
    fixtures = ['test.json']

    def test_menus_modify_primed_menu_items(self):

        # NOTE: Positional args used to ensure supplied args remain consistent
        @hooks.register('menus_modify_primed_menu_items')
        def modify_menu_items(
            menu_items, request, parent_context, parent_page, menu_instance,
            original_menu_instance, menu_tag, original_menu_tag, current_level,
            max_levels, use_specific, current_site, current_page,
            current_section_root_page, current_page_ancestor_ids,
            apply_active_classes, allow_repeating_parents,
            use_absolute_page_urls
        ):
            if original_menu_tag == 'main_menu' and current_level == 1:
                menu_items.append({
                    'href': 'https://rkh.co.uk',
                    'text': 'VISIT RKH.CO.UK',
                    'active_class': 'external',
                })
            return menu_items

        # Let's render the test homepage to see what happens!
        response = self.client.get('/')

        # unhook asap to prevent knock-on effects on failure
        del hooks._hooks['menus_modify_primed_menu_items']

        # If the the hook failed to receive any of the arguments defined
        # on `modify_menu_items` above, there will be an error
        self.assertEqual(response.status_code, 200)

        # There are 5 main menus being output, and because our hook only adds
        # the additional item to the first level of each of those, the
        # 'VISIT RKH.CO.UK' text should appear exactly 5 times
        self.assertContains(response, 'VISIT RKH.CO.UK', 5)

    def test_menus_modify_raw_menu_items(self):

        # NOTE: Positional args used to ensure supplied args remain consistent
        @hooks.register('menus_modify_raw_menu_items')
        def modify_menu_items(
            menu_items, request, parent_context, parent_page, menu_instance,
            original_menu_instance, menu_tag, original_menu_tag, current_level,
            max_levels, use_specific, current_site, current_page,
            current_section_root_page, current_page_ancestor_ids,
            apply_active_classes, allow_repeating_parents,
            use_absolute_page_urls
        ):
            if original_menu_tag == 'section_menu' and current_level == 1:
                """
                For the first level of section menus, add a copy of the first
                page to the end of the list
                """
                try:
                    menu_items.append(menu_items[0])
                except KeyError:
                    pass
            return menu_items

        # Let's render the 'about us' page to see what happens!
        response = self.client.get('/about-us/')

        # unhook asap to prevent knock-on effects on failure
        del hooks._hooks['menus_modify_raw_menu_items']

        # If the the hook failed to receive any of the arguments defined
        # on `modify_menu_items` above, there will be an error
        self.assertEqual(response.status_code, 200)

        # Test output reflects hook changes
        soup = BeautifulSoup(response.content, 'html5lib')
        section_menu_html = soup.find(id='section-menu-one-level').decode()
        # 'Call us' is a page link, so should no longer appear
        expected_html = """
        <div id="section-menu-one-level">
            <nav class="nav-section" role="navigation">
                <a href="/about-us/" class="ancestor section_root">About us</a>
                <ul>
                    <li class="active"><a href="/about-us/">Section home</a></li>
                    <li class=""><a href="/about-us/meet-the-team/">Meet the team</a></li>
                    <li class=""><a href="/about-us/our-heritage/">Our heritage</a></li>
                    <li class=""><a href="/about-us/mission-and-values/">Our mission and values</a></li>
                    <li class=""><a href="/about-us/meet-the-team/">Meet the team</a></li>
                </ul>
            </nav>
        </div>
        """
        self.assertHTMLEqual(section_menu_html, expected_html)

    def test_menus_modify_base_page_queryset(self):

        # NOTE: Positional args used to ensure supplied args remain consistent
        @hooks.register('menus_modify_base_page_queryset')
        def modify_page_queryset(
            queryset, request, parent_context, parent_page, menu_instance,
            original_menu_instance, menu_tag, original_menu_tag, current_level,
            max_levels, use_specific, current_site, current_page,
            current_section_root_page, current_page_ancestor_ids,
            apply_active_classes, allow_repeating_parents,
            use_absolute_page_urls
        ):
            """
            Nullify page queryset for 'flat menus'. Should result in only
            links to custom urls being rendered.
            """
            if menu_tag == 'flat_menu':
                queryset = queryset.none()
            return queryset

        # Let's render the test homepage to see what happens!
        response = self.client.get('/')

        # unhook asap to prevent knock-on effects on failure
        del hooks._hooks['menus_modify_base_page_queryset']

        # If the the hook failed to receive any of the arguments defined
        # on `modify_menu_items` above, there will be an error
        self.assertEqual(response.status_code, 200)

        # Test output reflects hook changes
        soup = BeautifulSoup(response.content, 'html5lib')
        contact_menu_html = soup.find(id='nav-contact').decode()
        # 'Call us' is a page link, so should no longer appear
        expected_html = """
        <div id="nav-contact">
            <div class="flat-menu contact no_heading">
                <ul>
                    <li class=""><a href="#advisor-chat">Chat to an advisor</a></li>
                    <li class=""><a href="#request-callback">Request a callback</a></li>
                </ul>
            </div>
        </div>
        """
        self.assertHTMLEqual(contact_menu_html, expected_html)

    def test_menus_modify_base_menuitem_queryset(self):

        # NOTE: Positional args used to ensure supplied args remain consistent
        @hooks.register('menus_modify_base_menuitem_queryset')
        def modify_menuitem_queryset(
            queryset, request, parent_context, parent_page, menu_instance,
            original_menu_instance, menu_tag, original_menu_tag, current_level,
            max_levels, use_specific, current_site, current_page,
            current_section_root_page, current_page_ancestor_ids,
            apply_active_classes, allow_repeating_parents,
            use_absolute_page_urls
        ):
            """
            Nullify menu items completely for all 'flat menus'. Should result
            in completely empty menus
            """
            if menu_tag == 'flat_menu':
                queryset = queryset.none()
            return queryset

        # Let's render the test homepage to see what happens!
        response = self.client.get('/')

        # unhook asap to prevent knock-on effects on failure
        del hooks._hooks['menus_modify_base_menuitem_queryset']

        # If the the hook failed to receive any of the arguments defined
        # on `modify_menu_items` above, there will be an error
        self.assertEqual(response.status_code, 200)

        # Test output reflects hook changes
        soup = BeautifulSoup(response.content, 'html5lib')
        contact_menu_html = soup.find(id='nav-contact').decode()
        # There should be no menu items, so just an empty div (no <ul>)
        expected_html = """
        <div id="nav-contact">
            <div class="flat-menu contact no_heading"></div>
        </div>
        """
        self.assertHTMLEqual(contact_menu_html, expected_html)



from django.utils.translation import ugettext_lazy as _

USE_SPECIFIC_OFF = 0
USE_SPECIFIC_AUTO = 1
USE_SPECIFIC_TOP_LEVEL = 2
USE_SPECIFIC_ALWAYS = 3
USE_SPECIFIC_CHOICES = (
    (USE_SPECIFIC_OFF, _("Off (most efficient)")),
    (USE_SPECIFIC_AUTO, _("Auto")),
    (USE_SPECIFIC_TOP_LEVEL, _("Top level")),
    (USE_SPECIFIC_ALWAYS, _("Always (least efficient)")),
)
MAX_LEVELS_CHOICES = (
    (1, _('1: No sub-navigation (flat)')),
    (2, _('2: Allow 1 level of sub-navigation')),
    (3, _('3: Allow 2 levels of sub-navigation')),
    (4, _('4: Allow 3 levels of sub-navigation')),
)



from django.core.exceptions import ValidationError
from django.test import TestCase
from django.test.client import RequestFactory
from wagtail.core.models import Page

from wagtailmenus.conf import settings
from wagtailmenus.models import (
    AbstractMenuItem, MainMenu, MainMenuItem, FlatMenu, FlatMenuItem
)


class MenuItemModelTestMixin:
    """
    Defines a generic set of tests to be run against either MainMenuItem or
    FlatMenuItem model classes
    """
    fixtures = ['test.json']
    menu_model = None
    menu_item_model = None

    def setUp(self):
        self.menu = self.menu_model.objects.all().first()
        self.page = Page.objects.all().first()

    def test_meta_class(self):
        opts = self.menu_item_model._meta
        self.assertEqual(opts.verbose_name, 'menu item')
        self.assertEqual(opts.verbose_name_plural, 'menu items')
        self.assertEqual(opts.ordering, ('sort_order',))

    def test_menu_text_returns_link_text_value_over_page_title(self):
        link_text = "i want to be menu text"
        obj = self.menu_item_model(
            menu=self.menu,
            link_page=self.page,
            link_text=link_text
        )
        self.assertNotEqual(self.page.title, link_text)
        self.assertEqual(obj.menu_text, link_text)

    def test_menu_text_returns_page_title_if_no_link_text_is_set(self):
        obj = self.menu_item_model(menu=self.menu, link_page=self.page)
        self.assertEqual(obj.menu_text, self.page.title)

    def test_menu_text_returns_blank_string_if_neither_link_text_or_link_page_are_set(self):
        obj = self.menu_item_model(menu=self.menu)
        self.assertEqual(obj.menu_text, '')

    def test_str_returns_same_value_as_menu_text(self):
        for obj in self.menu_item_model.objects.all():
            self.assertEqual(str(obj), obj.menu_text)

    def test_clean_errors_when_link_url_is_set_without_link_text(self):
        obj = self.menu_item_model(menu=self.menu, link_url='test/')
        self.assertRaisesMessage(
            ValidationError,
            "This field is required when linking to a custom URL",
            obj.clean)

    def test_clean_errors_when_both_link_url_and_link_page_are_set(self):
        obj = self.menu_item_model(
            menu=self.menu,
            link_url='test/',
            link_page=self.page
        )
        self.assertRaisesMessage(
            ValidationError,
            "Linking to both a page and custom URL is not permitted",
            obj.clean
        )

    def test_clean_errors_when_nethier_link_url_or_link_page_are_set(self):
        obj = self.menu_item_model(
            menu=self.menu,
            link_text='Test',
        )
        self.assertRaisesMessage(
            ValidationError,
            "Please choose an internal page or provide a custom URL",
            obj.clean
        )


class TestMainMenuItemGeneralMethods(MenuItemModelTestMixin, TestCase):
    """
    Runs the tests from MenuItemModelTestMixin for the MainMenuItem model
    """
    menu_model = MainMenu
    menu_item_model = MainMenuItem


class TestFlatMenuItemGeneralMethods(MenuItemModelTestMixin, TestCase):
    """
    Runs the tests from MenuItemModelTestMixin for the FlatMenuItem model
    """
    menu_model = FlatMenu
    menu_item_model = FlatMenuItem


class TestMenuItemsForRequest(TestCase):
    """
    Tests active classes rendered for menu items
    """
    def setUp(self):
        self.rf = RequestFactory()

    def test_menu_item_exact_match(self):
        menu_item = AbstractMenuItem()
        menu_item.link_url = '/some-page/'
        request = self.rf.get('/some-page/')

        active_class = menu_item.get_active_class_for_request(request)
        self.assertEqual(active_class, settings.ACTIVE_CLASS)

    def test_menu_item_with_query_params(self):
        menu_item = AbstractMenuItem()
        menu_item.link_url = '/some-page/?some_param=foo'
        request = self.rf.get('/some-page/?some_other_param=foo')

        active_class = menu_item.get_active_class_for_request(request)
        self.assertEqual(active_class, settings.ACTIVE_CLASS)

    def test_menu_item_with_hash_fragment(self):
        menu_item = AbstractMenuItem()
        menu_item.link_url = '/some-page/#some_fragment'
        request = self.rf.get('/some-page/#some_other_hash_fragment')

        active_class = menu_item.get_active_class_for_request(request)
        self.assertEqual(active_class, settings.ACTIVE_CLASS)

    def test_menu_item_ancestor(self):
        menu_item = AbstractMenuItem()
        menu_item.link_url = '/some-page/'
        request = self.rf.get('/some-page/some-child-page')

        active_class = menu_item.get_active_class_for_request(request)
        self.assertEqual(active_class, settings.ACTIVE_ANCESTOR_CLASS)

    def test_menu_item_with_netloc(self):
        menu_item = AbstractMenuItem()
        menu_item.link_url = 'https://example.com/some-page'
        request = self.rf.get('/some-page/')

        active_class = menu_item.get_active_class_for_request(request)
        self.assertEqual(active_class, '')



from django.template.loader import get_template, select_template

from wagtailmenus.conf import settings


def get_item_by_index_or_last_item(items, index):
    if items is None:
        return
    try:
        return items[index]
    except IndexError:
        return items[-1]


class DefinesSubMenuTemplatesMixin:
    # Use to specify a single sub menu template for all levels
    sub_menu_template_name = None
    # Use to specify sub menu templates for each level
    sub_menu_template_names = None

    def _get_specified_sub_menu_template_name(self, level):
        """
        Called by get_sub_menu_template(). Iterates through the various ways in
        which developers can specify potential sub menu templates for a menu,
        and returns the name of the most suitable template for the
        ``current_level``. Values are checked in the following order:

        1.  The ``sub_menu_template`` value passed to the template tag (if
            provided)
        2.  The most suitable template from the ``sub_menu_templates`` list
            passed to the template tag (if provided)
        3.  The ``sub_menu_template_name`` attribute set on the menu class (if
            set)
        4.  The most suitable template from a list of templates set as the
            ``sub_menu_template_names`` attribute on the menu class (if set)

        Parameters
        ----------
        level : int
            The 'current_level' value from the context, indicating the depth
            of sub menu being rendered as part of a multi-level menu. For sub
            menus, the value will always be greater than or equal to 2.

        Returns
        -------
        str or None
            A template name string (the path to a template in the file system),
            or None if no template has been 'specified'
        """
        ideal_index = level - 2
        return self._option_vals.sub_menu_template_name or \
            get_item_by_index_or_last_item(
                self._option_vals.sub_menu_template_names, ideal_index) or \
            self.sub_menu_template_name or \
            get_item_by_index_or_last_item(
                self.sub_menu_template_names, ideal_index)

    def get_sub_menu_template(self, level=2):
        if not hasattr(self, '_sub_menu_template_cache'):
            # Initialise cache for this menu instance
            self._sub_menu_template_cache = {}
        elif level in self._sub_menu_template_cache:
            # Return a cached template instance
            return self._sub_menu_template_cache[level]

        template_name = self._get_specified_sub_menu_template_name(level)
        if template_name:
            # A template was specified somehow
            template = get_template(template_name)
        else:
            # A template wasn't specified, so search the filesystem
            template = select_template(self.get_sub_menu_template_names())

        # Cache the template instance before returning
        self._sub_menu_template_cache[level] = template
        return template

    sub_menu_template = property(get_sub_menu_template)

    def get_sub_menu_template_names(self):
        """Return a list of template paths/names to search when rendering a
        sub menu for this menu instance. The list should beordered with most
        specific names first, since the first template found to exist will be
        used for rendering"""
        template_names = []
        menu_name = self.menu_short_name
        site = self._contextual_vals.current_site
        level = self._contextual_vals.current_level
        if settings.SITE_SPECIFIC_TEMPLATE_DIRS and site:
            hostname = site.hostname
            template_names.extend([
                "menus/%s/%s/level_%s.html" % (hostname, menu_name, level),
                "menus/%s/%s/sub_menu.html" % (hostname, menu_name),
                "menus/%s/%s_sub_menu.html" % (hostname, menu_name),
                "menus/%s/sub_menu.html" % hostname,
            ])
        template_names.extend([
            "menus/%s/level_%s.html" % (menu_name, level),
            "menus/%s/sub_menu.html" % menu_name,
            "menus/%s_sub_menu.html" % menu_name,
            settings.DEFAULT_SUB_MENU_TEMPLATE,
        ])
        return template_names

    def get_context_data(self, **kwargs):
        """
        Include the name of the sub menu template in the context. This is
        purely for backwards compatibility. Any sub menus rendered as part of
        this menu will call `sub_menu_template` on the original menu instance
        to get an actual `Template`
        """
        data = {}
        if self._contextual_vals.current_level == 1 and self.max_levels > 1:
            data['sub_menu_template'] = self.sub_menu_template.template.name
        data.update(kwargs)
        return super().get_context_data(**data)



import inspect

"""
This method below was copied from wagtail.wagtailcore.utils (v1.11) and allows
us to handle deprecation of method arguments in a way that doesn't swallow
`TypeError` exceptions raised further down the chain. Thank you @gasman!
"""


def accepts_kwarg(func, kwarg):
    """
    Determine whether the callable `func` has a signature that accepts the
    keyword argument `kwarg`
    """
    signature = inspect.signature(func)
    try:
        signature.bind_partial(**{kwarg: None})
        return True
    except TypeError:
        return False



from importlib import reload
from unittest.mock import call, patch

from django.test import TestCase, override_settings

from wagtailmenus.modeladmin import FlatMenuAdmin, MainMenuAdmin
from wagtailmenus import wagtail_hooks


class TestDisablingFlatMenusInWagtailCMS(TestCase):
    """
    Tests if modeladmin is registered based on settings.
    """
    @patch('wagtail.contrib.modeladmin.options.modeladmin_register')
    def test_modeladmin_registered_by_default(self, mock_modeladmin_register):
        reload(wagtail_hooks)
        self.assertIn(
            call(FlatMenuAdmin),
            mock_modeladmin_register.mock_calls
        )

    @override_settings(
        WAGTAILMENUS_FLAT_MENUS_EDITABLE_IN_WAGTAILADMIN=False,
    )
    @patch('wagtail.contrib.modeladmin.options.modeladmin_register')
    def test_modeladmin_not_registered_if_disabled(self, mock_modeladmin_register):
        """
        Disabling the 'Flat menu' via setting should prevent registering the
        FlatMenuAdmin, but MainMenuAdmin should still be registered.
        """
        reload(wagtail_hooks)
        self.assertNotIn(
            call(FlatMenuAdmin),
            mock_modeladmin_register.mock_calls
        )
        mock_modeladmin_register.assert_called_with(MainMenuAdmin)


class TestDisablingMainMenusInWagtailCMS(TestCase):

    @patch('wagtail.contrib.modeladmin.options.modeladmin_register')
    def test_modeladmin_registered_by_default(self, mock_modeladmin_register):
        reload(wagtail_hooks)
        self.assertIn(
            call(MainMenuAdmin),
            mock_modeladmin_register.mock_calls
        )

    @override_settings(
        WAGTAILMENUS_MAIN_MENUS_EDITABLE_IN_WAGTAILADMIN=False,
    )
    @patch('wagtail.contrib.modeladmin.options.modeladmin_register')
    def test_disable_main_menus(self, mock_modeladmin_register):
        """
        Disabling the 'Main menu' via setting should prevent registering the
        MainMenuAdmin, but FlatMenuAdmin should still be registered.
        """
        reload(wagtail_hooks)
        self.assertNotIn(
            call(MainMenuAdmin),
            mock_modeladmin_register.mock_calls
        )
        mock_modeladmin_register.assert_called_with(FlatMenuAdmin)



from distutils.version import LooseVersion

from django.conf import settings as django_settings
from django.utils.translation import ugettext_lazy as _
from wagtail.admin.edit_handlers import (
    FieldPanel, FieldRowPanel, InlinePanel, MultiFieldPanel,
    PageChooserPanel, ObjectList, TabbedInterface
)

from wagtailmenus.conf import settings


# ########################################################
# For menu models
# ########################################################

def _define_inlinepanel(relation_name, **kwargs):
    klass = InlinePanel
    defaults = {'label': _('menu items')}
    if 'condensedinlinepanel' in django_settings.INSTALLED_APPS:
        import condensedinlinepanel
        from condensedinlinepanel.edit_handlers import CondensedInlinePanel
        if LooseVersion(condensedinlinepanel.__version__) >= LooseVersion('0.3'):
            klass = CondensedInlinePanel
            defaults = {
                'heading': _('Menu items'),
                'label': _("Add new item"),
                'new_card_header_text': _("New item"),
            }
    defaults.update(kwargs)
    return klass(relation_name, **defaults)


def FlatMenuItemsInlinePanel(**kwargs):  # noqa
    """
    Returns either a ``InlinePanel`` or ``CondensedInlinePanel`` instance (
    depending on whether a sufficient version of `condensedinlinepanel` is
    installed) for editing menu items for a flat menu.

    Use in panel definitions like any standard panel class. Any supplied kwargs
    will be passed on as kwargs to the target class's __init__ method.
    """
    return _define_inlinepanel(
        relation_name=settings.FLAT_MENU_ITEMS_RELATED_NAME,
        **kwargs
    )


def MainMenuItemsInlinePanel(**kwargs):  # noqa
    """
    Returns either a ``InlinePanel`` or ``CondensedInlinePanel`` instance (
    depending on whether a sufficient version of `condensedinlinepanel` is
    installed) for editing menu items for a main menu.

    Use in panel definitions like any standard panel class. Any supplied kwargs
    will be passed on as kwargs to the target class's __init__ method.
    """
    return _define_inlinepanel(
        relation_name=settings.MAIN_MENU_ITEMS_RELATED_NAME,
        **kwargs
    )


main_menu_content_panels = (
    MainMenuItemsInlinePanel(),
)

flat_menu_content_panels = (
    MultiFieldPanel(
        heading=_("Menu details"),
        children=(
            FieldPanel('title'),
            FieldPanel('site'),
            FieldPanel('handle'),
            FieldPanel('heading'),
        ),
        classname="collapsible"
    ),
    FlatMenuItemsInlinePanel(),
)

menu_settings_panels = (
    MultiFieldPanel(
        heading=_('Render settings'),
        children=(
            FieldPanel('max_levels'),
            FieldPanel('use_specific')
        ),
    ),
)


# ########################################################
# For AbstractLinkPage
# ########################################################

linkpage_panels = [
    MultiFieldPanel([
        FieldPanel('title', classname="title"),
        PageChooserPanel('link_page'),
        FieldPanel('link_url'),
        FieldPanel('url_append'),
        FieldPanel('extra_classes'),
    ])
]

linkpage_tab = ObjectList(
    linkpage_panels, heading=_("Settings"), classname="settings"
)

linkpage_edit_handler = TabbedInterface([linkpage_tab])


# ########################################################
# For MenuPageMixin
# ########################################################

menupage_panel = MultiFieldPanel(
    heading=_("Advanced menu behaviour"),
    classname="collapsible collapsed",
    children=(
        FieldPanel('repeat_in_subnav'),
        FieldPanel('repeated_item_text'),
    )
)

menupage_settings_panels = [
    MultiFieldPanel(
        heading=_("Scheduled publishing"),
        classname="publishing",
        children=(
            FieldRowPanel((
                FieldPanel('go_live_at', classname="col6"),
                FieldPanel('expire_at', classname="col6"),
            )),
        )
    ),
    menupage_panel,
]

menupage_settings_tab = ObjectList(
    menupage_settings_panels, heading=_("Settings"), classname="settings"
)



from .constants import USE_SPECIFIC_AUTO

# NOTE: All supported app settings must be added here


# -------------------
# Admin / UI settings
# -------------------

ADD_EDITOR_OVERRIDE_STYLES = True

FLATMENU_MENU_ICON = 'list-ol'

FLAT_MENUS_EDITABLE_IN_WAGTAILADMIN = True

FLAT_MENUS_MODELADMIN_CLASS = 'wagtailmenus.modeladmin.FlatMenuAdmin'

FLAT_MENUS_HANDLE_CHOICES = None

MAINMENU_MENU_ICON = 'list-ol'

MAIN_MENUS_EDITABLE_IN_WAGTAILADMIN = True

MAIN_MENUS_MODELADMIN_CLASS = 'wagtailmenus.modeladmin.MainMenuAdmin'

USE_CONDENSEDINLINEPANEL = True


# ----------------------------------------------
# Default templates and template finder settings
# ----------------------------------------------

DEFAULT_CHILDREN_MENU_TEMPLATE = 'menus/children_menu.html'

DEFAULT_FLAT_MENU_TEMPLATE = 'menus/flat_menu.html'

DEFAULT_MAIN_MENU_TEMPLATE = 'menus/main_menu.html'

DEFAULT_SECTION_MENU_TEMPLATE = 'menus/section_menu.html'

DEFAULT_SUB_MENU_TEMPLATE = 'menus/sub_menu.html'

SITE_SPECIFIC_TEMPLATE_DIRS = False


# ------------------------------
# Default tag behaviour settings
# ------------------------------

DEFAULT_CHILDREN_MENU_MAX_LEVELS = 1

DEFAULT_SECTION_MENU_MAX_LEVELS = 2

DEFAULT_CHILDREN_MENU_USE_SPECIFIC = USE_SPECIFIC_AUTO

DEFAULT_SECTION_MENU_USE_SPECIFIC = USE_SPECIFIC_AUTO

DEFAULT_ADD_SUB_MENUS_INLINE = False

FLAT_MENUS_FALL_BACK_TO_DEFAULT_SITE_MENUS = False

GUESS_TREE_POSITION_FROM_PATH = True


# --------------------------------------
# Menu class and model override settings
# --------------------------------------

CHILDREN_MENU_CLASS = 'wagtailmenus.models.ChildrenMenu'

FLAT_MENU_MODEL = 'wagtailmenus.FlatMenu'

FLAT_MENU_ITEMS_RELATED_NAME = 'menu_items'

MAIN_MENU_MODEL = 'wagtailmenus.MainMenu'

MAIN_MENU_ITEMS_RELATED_NAME = 'menu_items'

SECTION_MENU_CLASS = 'wagtailmenus.models.SectionMenu'


# ----------------------
# Miscellaneous settings
# ----------------------

ACTIVE_CLASS = 'active'

ACTIVE_ANCESTOR_CLASS = 'ancestor'

PAGE_FIELD_FOR_MENU_ITEM_TEXT = 'title'

SECTION_ROOT_DEPTH = 3


# ----------
# Deprecated
# ----------



from django.test import TestCase

from wagtailmenus.models import ChildrenMenu
from wagtailmenus.tests import base, utils

Page = utils.get_page_model()


class ChildrenMenuTestCase(TestCase):

    def get_test_menu_instance(self):
        return ChildrenMenu(parent_page=Page(), max_levels=3, use_specific=1)


class TestGetSubMenuTemplateNames(
    ChildrenMenuTestCase, base.GetSubMenuTemplateNamesMethodTestCase
):
    """
    Tests ChildrenMenu.get_sub_menu_template_names() using common test cases
    from base.GetTemplateNamesMethodTestCase
    """
    expected_default_result_length = 4


class TestGetTemplateNames(
    ChildrenMenuTestCase, base.GetTemplateNamesMethodTestCase
):
    """
    Tests ChildrenMenu.get_template_names() using common test cases from
    base.GetTemplateNamesMethodTestCase
    """
    expected_default_result_length = 3



from django.template import RequestContext
from django.test.client import RequestFactory
from wagtailmenus.conf import constants
from wagtailmenus.models.menus import ContextualVals, OptionVals


SUB_MENU_TEMPLATE_LIST = (
    'menus/sub_menu_level_2.html',
    'menus/sub_menu_level_3.html',
)
SINGLE_ITEM_SUB_MENU_TEMPLATE_LIST = ('menus/sub_menu_level_2.html',)


def get_page_model():
    from wagtail.core.models import Page
    return Page


def get_site_model():
    from wagtail.core.models import Site
    return Site


def make_optionvals_instance(
    max_levels=2,
    use_specific=constants.USE_SPECIFIC_AUTO,
    apply_active_classes=True,
    allow_repeating_parents=True,
    use_absolute_page_urls=False,
    add_sub_menus_inline=False,
    parent_page=None,
    handle=None,
    template_name=None,
    sub_menu_template_name=None,
    sub_menu_template_names=None,
    extra={},
):
    return OptionVals(
        max_levels,
        use_specific,
        apply_active_classes,
        allow_repeating_parents,
        use_absolute_page_urls,
        add_sub_menus_inline,
        parent_page,
        handle,
        template_name,
        sub_menu_template_name,
        sub_menu_template_names,
        extra
    )


def make_contextualvals_instance(
    url='/',
    request=None,
    parent_context=None,
    current_site=None,
    current_level=1,
    original_menu_tag='',
    original_menu_instance=None,
    current_page=None,
    current_section_root_page=None,
    current_page_ancestor_ids=(),
):
    if request is None:
        rf = RequestFactory()
        request = rf.get(url)
    if parent_context is None:
        parent_context = RequestContext(request, {})
    return ContextualVals(
        parent_context,
        request,
        current_site,
        current_level,
        original_menu_tag,
        original_menu_instance,
        current_page,
        current_section_root_page,
        current_page_ancestor_ids,
    )



from django import forms
from django.utils.translation import ugettext_lazy as _
from wagtail.admin.forms import WagtailAdminModelForm, WagtailAdminPageForm

from wagtailmenus.conf import settings


class FlatMenuAdminForm(WagtailAdminModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if settings.FLAT_MENUS_HANDLE_CHOICES:
            self.fields['handle'] = forms.ChoiceField(
                label=self.fields['handle'].label,
                choices=settings.FLAT_MENUS_HANDLE_CHOICES
            )


class LinkPageAdminForm(WagtailAdminPageForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['title'].help_text = _(
            "By default, this will be used as the link text when appearing "
            "in menus."
        )



from django.test import TestCase
from django.core.exceptions import ValidationError
from wagtail.core.models import Site

from wagtailmenus.tests.models import LinkPage


class TestLinkPage(TestCase):
    fixtures = ['test.json']

    def setUp(self):
        # Create a few of link pages for testing
        site = Site.objects.select_related('root_page').get(is_default_site=True)
        self.site = site

        linkpage_to_page = LinkPage(
            title='Find out about Spiderman',
            link_page_id=30,
            url_append='?somevar=value'
        )
        site.root_page.add_child(instance=linkpage_to_page)

        # Check that the above page was saved and has field values we expect
        self.assertTrue(linkpage_to_page.id)
        self.assertTrue(linkpage_to_page.show_in_menus)
        self.assertTrue(linkpage_to_page.show_in_menus_custom())
        self.assertEqual(linkpage_to_page.get_sitemap_urls(), [])
        self.linkpage_to_page = linkpage_to_page

        linkpage_to_url = LinkPage(
            title='Do a google search',
            link_url="https://www.google.co.uk",
            url_append='?somevar=value',
            extra_classes='google external',
        )
        site.root_page.add_child(instance=linkpage_to_url)

        # Check that the above page was saved and has field values we expect
        self.assertTrue(linkpage_to_url.id)
        self.assertTrue(linkpage_to_url.show_in_menus)
        self.assertTrue(linkpage_to_url.show_in_menus_custom())
        self.assertEqual(linkpage_to_url.get_sitemap_urls(), [])
        self.linkpage_to_url = linkpage_to_url

        linkpage_to_non_routable_page = LinkPage(
            title='Go to this unroutable page',
            link_page_id=2,
            url_append='?somevar=value'
        )
        site.root_page.add_child(instance=linkpage_to_non_routable_page)
        self.linkpage_to_non_routable_page = linkpage_to_non_routable_page

    def test_url_methods(self):
        # When linking to a page
        self.assertEqual(
            self.linkpage_to_page.relative_url(self.site),
            '/superheroes/marvel-comics/spiderman/?somevar=value'
        )
        self.assertEqual(
            self.linkpage_to_page.full_url,
            'http://www.wagtailmenus.co.uk:8000/superheroes/marvel-comics/spiderman/?somevar=value'
        )

        # When linking to a non-routable page
        self.assertEqual(self.linkpage_to_non_routable_page.relative_url(self.site), '')
        self.assertEqual(self.linkpage_to_non_routable_page.full_url, '')

        # When linking to a custom url
        self.assertEqual(
            self.linkpage_to_url.relative_url(self.site), 'https://www.google.co.uk?somevar=value'
        )
        self.assertEqual(
            self.linkpage_to_url.full_url, 'https://www.google.co.uk?somevar=value'
        )

    def test_linkpage_visibility(self):
        page_link_html = (
            '<a href="/superheroes/marvel-comics/spiderman/?somevar=value">Find out about Spiderman</a>'
        )

        url_link_html = (
            '<li class="google external"><a href="https://www.google.co.uk?somevar=value">Do a google search</a></li>'
        )
        # When the target page is live, both the 'Spiderman' and 'Google' link
        # should appear
        response = self.client.get('/')
        self.assertContains(response, page_link_html, html=True)
        self.assertContains(response, url_link_html, html=True)

        # When the target page is not live, the linkpage shouldn't appear
        target_page = self.linkpage_to_page.link_page
        target_page.live = False
        target_page.save()
        response = self.client.get('/')
        self.assertNotContains(response, page_link_html, html=True)

        # When the target page isn't set to appear in menus, the linkpage
        # shouldn't appear
        target_page.live = True
        target_page.show_in_menus = False
        target_page.save()
        response = self.client.get('/')
        self.assertNotContains(response, page_link_html, html=True)

        # When the target page is 'expired', the linkpage shouldn't appear
        target_page.show_in_menus = True
        target_page.expired = True
        target_page.save()
        response = self.client.get('/')
        self.assertNotContains(response, page_link_html, html=True)

    def test_linkpage_clean(self):
        linkpage = self.linkpage_to_page
        linkpage.link_url = 'https://www.rkh.co.uk/'
        self.assertRaisesMessage(
            ValidationError,
            "Linking to both a page and custom URL is not permitted",
            linkpage.clean
        )

        linkpage.link_url = ''
        linkpage.link_page = None
        self.assertRaisesMessage(
            ValidationError,
            "Please choose an internal page or provide a custom URL",
            linkpage.clean
        )

        linkpage.link_page = linkpage
        self.assertRaisesMessage(
            ValidationError,
            "A link page cannot link to another link page",
            linkpage.clean
        )

    def test_linkpage_redirects_when_served(self):
        response = self.client.get('/find-out-about-spiderman/')
        self.assertRedirects(
            response,
            '/superheroes/marvel-comics/spiderman/?somevar=value'
        )





from django.test import TestCase
from django.core.management import call_command
from wagtail.core.models import Site
from wagtailmenus.conf import settings


class TestAutoPopulateMainMenus(TestCase):
    fixtures = ['test.json']

    def setUp(self):
        super().setUp()
        # Delete any existing main menus and their items
        self.model = settings.models.MAIN_MENU_MODEL
        self.model.objects.all().delete()

    def test_without_home_links(self):
        call_command('autopopulate_main_menus', add_home_links=False)
        site = Site.objects.all().first()
        menu = self.model.get_for_site(site)
        menu_items = menu.get_menu_items_manager()

        # Confirm that there are menu items
        self.assertTrue(menu_items.count() == 5)

        # Confirm that the first item is NOT a home page link
        self.assertFalse(menu_items.first().menu_text == 'Home')

        # Confirm that the the menu items remain unaffected if the command
        # happens to be run more than once
        call_command('autopopulate_main_menus', add_home_links=False)
        menu = self.model.get_for_site(site)
        menu_items2 = menu.get_menu_items_manager()
        self.assertEqual(list(menu_items.all()), list(menu_items2.all()))

    def test_with_home_links(self):
        call_command('autopopulate_main_menus', add_home_links=True)
        site = Site.objects.all().first()
        menu = self.model.get_for_site(site)
        menu_items = menu.get_menu_items_manager()

        # Confirm that there are menu items
        self.assertTrue(menu_items.count() == 6)

        # Confirm that the first item is a home page link
        self.assertTrue(menu_items.first().menu_text == 'Home')

        # Confirm that the the menu items remain unaffected if the command
        # happens to be run more than once
        call_command('autopopulate_main_menus', add_home_links=True)
        menu = self.model.get_for_site(site)
        menu_items2 = menu.get_menu_items_manager()
        self.assertEqual(list(menu_items.all()), list(menu_items2.all()))



from django.test import TestCase, override_settings

from wagtailmenus.tests import utils


class GetSubMenuTemplateNamesMethodTestCase(TestCase):
    """
    A 'base' test case for a testing a menu class's
    get_sub_menu_template_names() method.
    """

    expected_default_result_length = None

    def get_test_menu_instance(self):
        # Must be overridden to supply a menu instance of the right type
        raise NotImplementedError()

    def get_test_site_instance(self):
        Site = utils.get_site_model()
        Page = utils.get_page_model()
        return Site.objects.all().first() or Site.objects.create(
            id=1,
            hostname='wagtailmenus.co.uk',
            root_page=Page.objects.first(),
        )

    def test_site_specific_templates_not_returned_by_default(self):
        menu = self.get_test_menu_instance()
        site = self.get_test_site_instance()

        menu._contextual_vals = utils.make_contextualvals_instance(
            current_site=site
        )
        result = menu.get_sub_menu_template_names()
        self.assertEqual(len(result), self.expected_default_result_length)
        for val in result:
            self.assertFalse(site.hostname in val)

    @override_settings(WAGTAILMENUS_SITE_SPECIFIC_TEMPLATE_DIRS=True)
    def test_specific_templates_returned_if_setting_is_true_and_current_site_is_in_contextual_vals(self):
        menu = self.get_test_menu_instance()
        site = self.get_test_site_instance()

        menu._contextual_vals = utils.make_contextualvals_instance(
            current_site=site
        )
        result = menu.get_sub_menu_template_names()
        self.assertGreater(len(result), self.expected_default_result_length)
        for val in result[:2]:
            self.assertTrue(site.hostname in val)

    @override_settings(WAGTAILMENUS_SITE_SPECIFIC_TEMPLATE_DIRS=True)
    def test_specific_templates_not_returned_if_current_site_not_in_contextual_vals(self):
        menu = self.get_test_menu_instance()
        site = self.get_test_site_instance()

        menu._contextual_vals = utils.make_contextualvals_instance(
            current_site=None
        )
        result = menu.get_sub_menu_template_names()
        self.assertEqual(len(result), self.expected_default_result_length)
        for val in result:
            self.assertTrue(site.hostname not in val)


class GetTemplateNamesMethodTestCase(TestCase):

    expected_default_result_length = 3

    def get_test_menu_instance(self):
        # Must be overridden to supply a menu instance of the right type
        raise NotImplementedError()

    def get_test_site_instance(self):
        Site = utils.get_site_model()
        Page = utils.get_page_model()
        return Site.objects.all().first() or Site.objects.create(
            id=1,
            hostname='wagtailmenus.co.uk',
            root_page=Page.objects.first(),
        )

    def test_site_specific_templates_not_returned_by_default(self):
        menu = self.get_test_menu_instance()
        site = self.get_test_site_instance()

        menu._contextual_vals = utils.make_contextualvals_instance(
            url='/', current_site=site
        )
        result = menu.get_template_names()
        self.assertEqual(len(result), self.expected_default_result_length)
        for val in result:
            self.assertFalse(site.hostname in val)

    @override_settings(WAGTAILMENUS_SITE_SPECIFIC_TEMPLATE_DIRS=True)
    def test_site_specific_templates_returned_if_setting_is_true_and_current_site_is_in_contextual_vals(self):
        menu = self.get_test_menu_instance()
        site = self.get_test_site_instance()

        menu._contextual_vals = utils.make_contextualvals_instance(
            url='/', current_site=site
        )
        result = menu.get_template_names()
        self.assertGreater(len(result), self.expected_default_result_length)
        for val in result[:2]:
            self.assertTrue(site.hostname in val)

    @override_settings(WAGTAILMENUS_SITE_SPECIFIC_TEMPLATE_DIRS=True)
    def test_specific_templates_not_returned_if_current_site_not_in_contextual_vals(self):
        menu = self.get_test_menu_instance()
        site = self.get_test_site_instance()

        menu._contextual_vals = utils.make_contextualvals_instance(
            url='/', current_site=None
        )
        result = menu.get_template_names()
        self.assertEqual(len(result), self.expected_default_result_length)
        for val in result:
            self.assertTrue(site.hostname not in val)



from wagtail.core import hooks
from wagtail.contrib.modeladmin.options import modeladmin_register

from wagtailmenus.conf import settings
from wagtailmenus.modeladmin import ( # noqa
    MainMenuAdmin, FlatMenuAdmin, FlatMenuButtonHelper
)

if settings.MAIN_MENUS_EDITABLE_IN_WAGTAILADMIN:
    modeladmin_register(settings.objects.MAIN_MENUS_MODELADMIN_CLASS)


if settings.FLAT_MENUS_EDITABLE_IN_WAGTAILADMIN:
    modeladmin_register(settings.objects.FLAT_MENUS_MODELADMIN_CLASS)


@hooks.register('before_serve_page')
def wagtailmenu_params_helper(page, request, serve_args, serve_kwargs):
    section_root = request.site.root_page.get_descendants().ancestor_of(
        page, inclusive=True
    ).filter(depth__exact=settings.SECTION_ROOT_DEPTH).first()
    if section_root:
        section_root = section_root.specific
    ancestor_ids = page.get_ancestors().filter(
        depth__gte=settings.SECTION_ROOT_DEPTH
    ).values_list('id', flat=True)
    request.META.update({
        'WAGTAILMENUS_CURRENT_SECTION_ROOT': section_root,
        'WAGTAILMENUS_CURRENT_PAGE': page,
        'WAGTAILMENUS_CURRENT_PAGE_ANCESTOR_IDS': ancestor_ids,
    })



class RemovedInWagtailMenus213Warning(DeprecationWarning):
    pass


removed_in_next_version_warning = RemovedInWagtailMenus213Warning


class RemovedInWagtailMenus3Warning(PendingDeprecationWarning):
    pass


class RemovedInWagtailMenus31Warning(PendingDeprecationWarning):
    pass



from django.test import TestCase

from wagtailmenus.models import SectionMenu
from wagtailmenus.tests import base, utils

Page = utils.get_page_model()


class SectionMenuTestCase(TestCase):

    def get_test_menu_instance(self):
        return SectionMenu(root_page=Page(), max_levels=3, use_specific=1)


class TestSectionMenuGeneralMethods(SectionMenuTestCase):

    def test_get_from_collected_values_is_not_implemented(self):
        # Non model-based menus use create_from_collected_values() instead of
        # get_from_collected_values(), because there's no 'getting' involved.
        menu = self.get_test_menu_instance()
        with self.assertRaises(NotImplementedError):
            menu.get_from_collected_values(None, None)


class TestGetSubMenuTemplateNames(
    SectionMenuTestCase, base.GetSubMenuTemplateNamesMethodTestCase
):
    """
    Tests SectionMenu.get_sub_menu_template_names() using common test cases
    from base.GetTemplateNamesMethodTestCase
    """
    expected_default_result_length = 4


class TestGetTemplateNames(
    SectionMenuTestCase, base.GetTemplateNamesMethodTestCase
):
    """
    Tests SectionMenu.get_template_names() using common test cases from
    base.GetTemplateNamesMethodTestCase
    """
    expected_default_result_length = 3



from django.test import TestCase

from wagtailmenus.models import FlatMenu
from wagtailmenus.tests import base, utils

Page = utils.get_page_model()
Site = utils.get_site_model()


class FlatMenuTestCase(TestCase):
    """A base TestCase class for testing FlatMenu model class methods"""

    @staticmethod
    def create_test_menus_for_site(site, count=3, set_option_vals=False):
        for i in range(1, count + 1):
            obj = FlatMenu.objects.create(
                site=site, handle='test-%s' % i, title='Test Menu %s' % i
            )
            if set_option_vals:
                obj._option_vals = utils.make_optionvals_instance()
            yield obj

    def setUp(self):
        self.site = Site.objects.get(is_default_site=True)
        self.menus = tuple(
            self.create_test_menus_for_site(self.site, set_option_vals=True)
        )

    def get_test_menu_instance(self):
        return self.menus[0]


class TestGetForSite(FlatMenuTestCase):
    """Unit tests for AbstractFlatMenu.get_for_site()"""
    def setUp(self):
        super().setUp()
        self.default_site = self.site
        self.not_default_site = Site.objects.create(
            hostname='test2.com',
            root_page=Page.objects.all().first(),
            is_default_site=False,
        )
        self.not_default_site_menus = tuple(
            self.create_test_menus_for_site(self.not_default_site)
        )

    def test_returns_none_if_no_match_for_supplied_site_and_fall_back_to_default_site_menus_is_false(self):
        test_handle = 'test-1'

        FlatMenu.objects.filter(site=self.not_default_site, handle=test_handle).delete()

        with self.assertNumQueries(1):
            result = FlatMenu.get_for_site(
                handle=test_handle,
                site=self.not_default_site,
                fall_back_to_default_site_menus=False
            )
        self.assertIs(result, None)

    def test_returns_menu_for_default_site_if_no_match_for_supplied_site_and_fall_back_to_default_site_menus_is_true(self):
        test_handle = 'test-1'

        FlatMenu.objects.filter(site=self.not_default_site, handle=test_handle).delete()

        expected_result = FlatMenu.objects.get(
            site=self.default_site, handle=test_handle)

        with self.assertNumQueries(1):
            result = FlatMenu.get_for_site(
                handle=test_handle,
                site=self.not_default_site,
                fall_back_to_default_site_menus=True
            )
        self.assertEqual(result, expected_result)

    def test_returns_provided_site_matches_over_default_site_matches(self):
        for handle in ('test-1', 'test-2', 'test-3'):
            with self.assertNumQueries(1):
                result = FlatMenu.get_for_site(
                    handle=handle,
                    site=self.not_default_site,
                    fall_back_to_default_site_menus=True
                )
            self.assertEqual(result.site_id, self.not_default_site.id)


class TestGetSubMenuTemplateNames(
    FlatMenuTestCase, base.GetSubMenuTemplateNamesMethodTestCase
):
    """
    Tests FlatMenu.get_sub_menu_template_names() using common test cases
    from base.GetTemplateNamesMethodTestCase
    """
    expected_default_result_length = 9


class TestGetTemplateNames(
    FlatMenuTestCase, base.GetTemplateNamesMethodTestCase
):
    """
    Tests FlatMenu.get_template_names() using common test cases from
    base.GetTemplateNamesMethodTestCase
    """
    expected_default_result_length = 10



import warnings
from unittest import mock
from django.test import TestCase

from wagtailmenus.conf import constants
from wagtailmenus.models import MainMenu, MainMenuItem
from wagtailmenus.tests import base, utils

Page = utils.get_page_model()


class MainMenuTestCase(TestCase):
    """A base TestCase class for testing MainMenu model class methods"""

    fixtures = ['test.json']

    def get_random_menu_instance_with_opt_vals_set(self):
        obj = MainMenu.objects.order_by('?').first()
        obj._option_vals = utils.make_optionvals_instance()
        return obj

    def get_test_menu_instance(self):
        return MainMenu.objects.first()


class TestMainMenuGeneralMethods(MainMenuTestCase):

    def test_create_from_collected_values_is_not_implemented(self):
        # Model-based menus use get_from_collected_values() instead of
        # create_from_collected_values(), because existing objects are reused,
        # rather than recreated each time
        menu = self.get_test_menu_instance()
        with self.assertRaises(NotImplementedError):
            menu.create_from_collected_values(None, None)


class TestTopLevelItems(MainMenuTestCase):

    # ------------------------------------------------------------------------
    # MainMenu.top_level_items()
    # ------------------------------------------------------------------------

    def test_setting_and_clearing_of_cache_values(self):
        menu = MainMenu.objects.get(pk=1)
        # This menu has a `use_specific` value of 1 (AUTO)
        self.assertEqual(menu.use_specific, 1)

        # So, the top-level items it returns should all just be custom links
        # or have a `link_page` that is just a vanilla Page object.
        for item in menu.top_level_items:
            self.assertTrue(item.link_page is None or type(item.link_page) is Page)

        # After being called once, top_level_options should be cached, so
        # accessing it again shouldn't trigger any database queries
        with self.assertNumQueries(0):
            for item in menu.top_level_items:
                self.assertTrue(item.link_page or item.link_url)

        # Lets change `use_specific` to 2 (TOP_LEVEL)
        menu.set_use_specific(2)

        # The above should clear the cached value, so if we call
        # top_level_items again, we should get a fresh list of specific pages
        for item in menu.top_level_items:
            self.assertTrue(item.link_page is None or type(item.link_page) is not Page)

        # Lets change `use_specific` to 0 (OFF) again
        menu.set_use_specific(0)

        # Because the work has already been done to fetch specific items, even
        # though we might not need specific items, the cached value should
        # remain in-tact, with no more queries being triggered on recall.
        with self.assertNumQueries(0):
            for item in menu.top_level_items:
                assert item.link_page is None or type(item.link_page) is not Page

    def test_method_initiates_one_query_when_no_menu_items_link_to_pages(self):
        # First, let's replace any menu items that link to pages with links
        # to custom urls
        for i, item in enumerate(
            MainMenu.objects.get(pk=1).get_menu_items_manager().all()
        ):
            if item.link_page_id:
                item.link_page = None
                item.link_url = '/test/{}/'.format(i)
                item.save()

        menu = MainMenu.objects.get(pk=1)
        with self.assertNumQueries(1):
            menu.get_top_level_items()

    def test_method_initiates_two_queries_when_vanilla_page_data_is_required(self, menu_obj=None):
        menu = menu_obj or MainMenu.objects.get(pk=1)
        menu.use_specific = constants.USE_SPECIFIC_AUTO
        with self.assertNumQueries(2):
            menu.get_top_level_items()


class TestPagesForDisplay(MainMenuTestCase):

    # ------------------------------------------------------------------------
    # MainMenu.pages_for_display()
    # ------------------------------------------------------------------------

    def test_setting_and_clearing_of_cache_values(self):
        menu = MainMenu.objects.get(pk=1)
        # This menu has a `use_specific` value of 1 (AUTO)
        self.assertEqual(menu.use_specific, 1)
        # And a `max_levels` value of 2
        self.assertEqual(menu.max_levels, 2)

        # So, every page returned by `pages_for_display` should just be a
        # vanilla Page object, that is live, not expired and meant to
        # appear in menus
        for p in menu.pages_for_display:
            self.assertEqual(type(p), Page)
            self.assertTrue(p.live)
            self.assertFalse(p.expired)
            self.assertTrue(p.show_in_menus)

        # Their should be 6 pages total
        self.assertEqual(len(menu.pages_for_display), 6)

        # After being called once, pages_for_display should be cached, so
        # accessing it again shouldn't trigger any database queries
        with self.assertNumQueries(0):
            for p in menu.pages_for_display:
                self.assertTrue(p.title)

        # Lets change `use_specific` to 3 (ALWAYS)
        menu.set_use_specific(3)

        # The above should clear the cached value, so if we call
        # pages_for_display again, we should get a fresh list of specific pages
        for p in menu.pages_for_display:
            self.assertNotEqual(type(p), Page)

        # Lets change `max_levels` to 1
        menu.set_max_levels(1)

        # The above should clear the cached value, so if we call
        # pages_for_display again, we it should provide an empty queryset
        self.assertQuerysetEqual(menu.pages_for_display, Page.objects.none())

        # Lets change `max_levels` to 3
        menu.set_max_levels(3)

        # The above should cleare the cached value, and now pages_for_display
        # should reurn 12 pages total to display
        self.assertEqual(len(menu.pages_for_display), 9)

        # As with `top_level_items` Even is we set `use_specific` to a lower
        # value, the cached `pages_for_display` value will not be cleared, as
        # it specific is better than not.
        menu.set_use_specific(1)

        # Because the work has already been done to fetch specific pages, even
        # though we might not need specific items, the cached value should
        # remain in-tact, with no more queries being triggered on recall.
        with self.assertNumQueries(0):
            for p in menu.pages_for_display:
                assert type(p) is not Page


class TestAddMenuItemsForPages(MainMenuTestCase):

    # ------------------------------------------------------------------------
    # MainMenu.add_menu_items_for_pages()
    # ------------------------------------------------------------------------

    def test_add_menu_items_for_pages(self):
        menu = MainMenu.objects.get(pk=1)
        # The current number of menu items is 6
        self.assertEqual(menu.get_menu_items_manager().count(), 6)

        # 'Superheroes' has 2 children: 'D.C. Comics' & 'Marvel Comics'
        superheroes_page = Page.objects.get(title="Superheroes")
        children_of_superheroes = superheroes_page.get_children()
        self.assertEqual(children_of_superheroes.count(), 2)

        # Use 'add_menu_items_for_pages' to add pages for the above pages
        menu.add_menu_items_for_pages(children_of_superheroes)

        # The number of menu items should now be 8
        self.assertEqual(menu.get_menu_items_manager().count(), 8)

        # Evaluate menu items to a list
        menu_items = list(menu.get_menu_items_manager().all())

        # The last item should be a link to the 'D.C. Comics' page, and the
        # sort_order on the item should be 7
        dc_item = menu_items.pop()
        self.assertEqual(dc_item.link_page.title, 'D.C. Comics')
        self.assertEqual(dc_item.sort_order, 7)

        # The '2nd to last' item should be a link to the 'Marvel Comics' page,
        # and the sort_order on the item should be 6
        marvel_item = menu_items.pop()
        self.assertEqual(marvel_item.link_page.title, 'Marvel Comics')
        self.assertEqual(marvel_item.sort_order, 6)


class TestGetSpecifiedSubMenuTemplateName(MainMenuTestCase):

    # ------------------------------------------------------------------------
    # MainMenu._get_specified_sub_menu_template_name()
    # (inherited from mixins.DefinesSubMenuTemplatesMixin)
    # ------------------------------------------------------------------------

    def test_returns_none_if_no_templates_specified(self):
        menu = self.get_random_menu_instance_with_opt_vals_set()
        self.assertEqual(
            menu._get_specified_sub_menu_template_name(level=2), None
        )
        self.assertEqual(
            menu._get_specified_sub_menu_template_name(level=3), None
        )
        self.assertEqual(
            menu._get_specified_sub_menu_template_name(level=4), None
        )

    def test_returns_last_template_when_no_template_specified_for_level(self):
        menu = MainMenu.objects.all().first()
        menu._option_vals = utils.make_optionvals_instance(
            sub_menu_template_names=('single_template.html',)
        )
        self.assertEqual(
            menu._get_specified_sub_menu_template_name(level=2),
            'single_template.html'
        )
        self.assertEqual(
            menu._get_specified_sub_menu_template_name(level=3),
            'single_template.html'
        )

    def test_preference_order_of_specified_values(self):
        menu = MainMenu.objects.all().first()
        menu._option_vals = utils.make_optionvals_instance(
            sub_menu_template_name='single_template_as_option.html',
            sub_menu_template_names=('option_one.html', 'option_two.html')
        )
        menu.sub_menu_template_name = 'single_template_as_attr.html'
        menu.sub_menu_template_names = utils.SUB_MENU_TEMPLATE_LIST

        # While both 'sub_menu_template_name' and 'sub_menu_template_names' are
        # specified as option values, the 'sub_menu_template_name' value will
        # be preferred
        self.assertEqual(
            menu._get_specified_sub_menu_template_name(level=4),
            'single_template_as_option.html'
        )

        # If only 'sub_menu_template_names' is specified as an option value,
        # that will be preferred
        menu._option_vals = utils.make_optionvals_instance(
            sub_menu_template_name=None,
            sub_menu_template_names=('option_one.html', 'option_two.html')
        )
        self.assertEqual(
            menu._get_specified_sub_menu_template_name(level=4),
            'option_two.html',
        )

        # If no templates have been specified via options, the
        # 'sub_menu_template_name' attribute is preferred
        menu._option_vals = utils.make_optionvals_instance(
            sub_menu_template_name=None,
            sub_menu_template_names=None
        )
        self.assertEqual(
            menu._get_specified_sub_menu_template_name(level=4),
            'single_template_as_attr.html'
        )

        # If the 'sub_menu_template_name' attribute is None, the method
        # should prefer the 'sub_menu_template_names' attribute
        menu.sub_menu_template_name = None
        self.assertEqual(
            menu._get_specified_sub_menu_template_name(level=4),
            menu.sub_menu_template_names[1]
        )


class TestGetSubMenuTemplateNames(
    MainMenuTestCase, base.GetSubMenuTemplateNamesMethodTestCase
):
    """
    Tests MainMenu.get_sub_menu_template_names() using common test cases
    from base.GetTemplateNamesMethodTestCase
    """
    expected_default_result_length = 4


class TestGetTemplateNames(
    MainMenuTestCase, base.GetTemplateNamesMethodTestCase
):
    """
    Tests MainMenu.get_template_names() using common test cases from
    base.GetTemplateNamesMethodTestCase
    """
    expected_default_result_length = 3


def mock_relative_url_method(self, site=None):
    return ''


class TestMenuItemDeprecationWarning(MainMenuTestCase):

    @mock.patch.object(MainMenuItem, 'relative_url', mock_relative_url_method)
    def test_warns_when_custom_menuitem_relative_url_does_not_accept_request(self):
        menu = self.get_test_menu_instance()
        menu._option_vals = utils.make_optionvals_instance()
        menu._contextual_vals = utils.make_contextualvals_instance(url='/test')
        menu.request = menu._contextual_vals.request

        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")
            menu.get_menu_items_for_rendering()
        self.assertEqual(
            str(w[0].message),
            "The relative_url() method on custom MenuItem classes must accept "
            "a 'request' keyword argument. Please update the method signature "
            "on your MainMenuItem class. It will be mandatory in Wagtail 2.13."
        )
